package fitconnect.api;

import com.nimbusds.jose.jwk.RSAKey;
import fitconnect.api.domain.auth.OAuthToken;
import fitconnect.api.domain.model.metadata.Metadata;
import fitconnect.api.domain.model.metadata.attachment.Attachment;
import fitconnect.api.domain.model.metadata.data.Data;
import fitconnect.api.domain.validation.ValidationResult;

import java.util.Optional;

/**
 * A technical system that accepts submissions on the administration side.
 */
public interface Subscriber {

    /**
     * Authenticates the {@link Subscriber} against the FitConnect-Auth-API and retrieves an {@link OAuthToken}.
     *
     * @param clientId a unique client identifier
     * @param clientSecret the applications secret key
     * @param scope 1 or more client scopes that determine if a submission is accepted by the client
     * @return {@link OAuthToken}, is empty if an error occurred
     */
    Optional<OAuthToken> retrieveOAuthToken(final String clientId, final String clientSecret, final String... scope);

    /**
     * Decrypts the JWE-encrypted submission data payload (json or xml).
     *
     * @param privateKey the private key to decrypt the {@link Data} payload
     * @param encryptedData the {@link Data} that is decrypted
     * @return the decrypted {@link Data}, is empty if en error occurred
     */
    Optional<Data> decryptSubmissionData(final RSAKey privateKey, final Data encryptedData);

    /**
     * Decrypts the JWE-encrypted submission attachment binary data.
     *
     * @param privateKey the private key to decrypt the {@link Attachment}
     * @param encryptedAttachment the JWE-encrypted {@link Attachment} that should be decrypted
     * @return the decrypted {@link Attachment}, is empty if en error occurred
     */
    Optional<Attachment> decryptAttachment(final RSAKey privateKey, final Attachment encryptedAttachment);

    /**
     * Validates the {@link Metadata} structure against a given JSON-schema to ensure its correctness.
     *
     * @param metadata the {@link Metadata} object that is validated
     * @param jsonSchema the schema in JSON-format that the given {@link Metadata} structure is compared with
     * @return a {@link ValidationResult}, contains error if the {@link Metadata} doesn't match the schema
     */
    ValidationResult validateMetadataSchema(final Metadata metadata, final String jsonSchema);

    /**
     * Validates the hash-values of the {@link Metadata}s {@link Data} and {@link Attachment} in order to guarantee
     * the integrity of the transmitted information.
     *
     * @param metadata the {@link Metadata} object including the hashes that where created by the {@link Sender}
     * @return a {@link ValidationResult}, that contains errors if the hash-values of the transmitted data are not valid
     */
    ValidationResult validateMetadataHashValues(final Metadata metadata);
}
